@@ -4,11 +4,18 @@
 The OsmApi module is a wrapper for the OpenStreetMap API.
 As such it provides an easy access to the functionality of the API.
 
+You can find this module [on PyPI](https://pypi.python.org/pypi/osmapi)
+or [on GitHub](https://github.com/metaodi/osmapi).
+
+Find all information about changes of the different versions of this module
+[in the CHANGELOG](https://github.com/metaodi/osmapi/blob/master/CHANGELOG.md).
+
+
 ## Notes:
 
 * **dictionary keys** are _unicode_
 * **changeset** is _integer_
-* *version** is _integer_
+* **version** is _integer_
 * **tag** is a _dictionary_
 * **timestamp** is _unicode_
 * **user** is _unicode_
@@ -68,6 +75,14 @@ class ApiError(Exception):
         )
 
 
+class AlreadySubscribedApiError(ApiError):
+    pass
+
+
+class NotSubscribedApiError(ApiError):
+    pass
+
+
 class OsmApi:
     """
     Main class of osmapi, instanciate this class to use osmapi
@@ -165,7 +180,7 @@ class OsmApi:
         self._CurrentChangesetId = 0
 
         # Http connection
-        self._conn = httplib.HTTPConnection(self._api, 80)
+        self._conn = self._get_http_connection()
 
     def __del__(self):
         if self._changesetauto:
@@ -1008,7 +1023,7 @@ class OsmApi:
     # Changeset                                      #
     ##################################################
 
-    def ChangesetGet(self, ChangesetId):
+    def ChangesetGet(self, ChangesetId, include_discussion=False):
         """
         Returns changeset with `ChangesetId` as a dict:
 
@@ -1020,6 +1035,7 @@ class OsmApi:
                 'created_at': timestamp of creation of this changeset
                 'closed_at': timestamp when changeset was closed
                 'comments_count': amount of comments
+                'discussion': [] list of comment dict (-> `include_discussion`)
                 'max_lon': maximum longitude of changes in this changeset
                 'max_lat': maximum latitude of changes in this changeset
                 'min_lon': minimum longitude of changes in this changeset
@@ -1029,8 +1045,14 @@ class OsmApi:
             }
 
         `ChangesetId` is the unique identifier of a changeset.
+
+        If `include_discussion` is set to `True` the changeset discussion
+        will be available in the result.
         """
-        data = self._get("/api/0.6/changeset/"+str(ChangesetId))
+        path = "/api/0.6/changeset/"+str(ChangesetId)
+        if (include_discussion):
+            path = path + "?include_discussion=true"
+        data = self._get(path)
         data = xml.dom.minidom.parseString(data)
         data = data.getElementsByTagName("osm")[0]
         data = data.getElementsByTagName("changeset")[0]
@@ -1211,6 +1233,119 @@ class OsmApi:
             result[tmpCS["id"]] = tmpCS
         return result
 
+    def ChangesetComment(self, ChangesetId, comment):
+        """
+        Adds a comment to the changeset `ChangesetId`
+
+        `comment` should be a string.
+
+        Returns the updated `ChangesetData` dict:
+
+            #!python
+            {
+                'id': id of Changeset,
+                'open': True|False, wheter or not this changeset is open
+                'tag': {} dict of tags,
+                'created_at': timestamp of creation of this changeset
+                'closed_at': timestamp when changeset was closed
+                'comments_count': amount of comments
+                'max_lon': maximum longitude of changes in this changeset
+                'max_lat': maximum latitude of changes in this changeset
+                'min_lon': minimum longitude of changes in this changeset
+                'min_lat': minimum longitude of changes in this changeset
+                'user': username of user that created this changeset,
+                'uid': id of user that created this changeset,
+            }
+
+        """
+        params = urllib.urlencode({'text': comment})
+        data = self._post(
+            "/api/0.6/changeset/"+str(ChangesetId)+"/comment",
+            params
+        )
+        data = xml.dom.minidom.parseString(data)
+        data = data.getElementsByTagName("osm")[0]
+        data = data.getElementsByTagName("changeset")[0]
+        return self._DomParseChangeset(data)
+
+    def ChangesetSubscribe(self, ChangesetId):
+        """
+        Subcribe to the changeset discussion of changeset `ChangesetId`.
+
+        The user will be informed about new comments (i.e. receive an email).
+
+        Returns the updated `ChangesetData` dict:
+
+            #!python
+            {
+                'id': id of Changeset,
+                'open': True|False, wheter or not this changeset is open
+                'tag': {} dict of tags,
+                'created_at': timestamp of creation of this changeset
+                'closed_at': timestamp when changeset was closed
+                'comments_count': amount of comments
+                'max_lon': maximum longitude of changes in this changeset
+                'max_lat': maximum latitude of changes in this changeset
+                'min_lon': minimum longitude of changes in this changeset
+                'min_lat': minimum longitude of changes in this changeset
+                'user': username of user that created this changeset,
+                'uid': id of user that created this changeset,
+            }
+        """
+        try:
+            data = self._post(
+                "/api/0.6/changeset/"+str(ChangesetId)+"/subscribe",
+                None
+            )
+        except ApiError as e:
+            if e.status == 409:
+                raise AlreadySubscribedApiError(e.status, e.reason, e.payload)
+            else:
+                raise
+        data = xml.dom.minidom.parseString(data)
+        data = data.getElementsByTagName("osm")[0]
+        data = data.getElementsByTagName("changeset")[0]
+        return self._DomParseChangeset(data)
+
+    def ChangesetUnsubscribe(self, ChangesetId):
+        """
+        Subcribe to the changeset discussion of changeset `ChangesetId`.
+
+        The user will be informed about new comments (i.e. receive an email).
+
+        Returns the updated `ChangesetData` dict:
+
+            #!python
+            {
+                'id': id of Changeset,
+                'open': True|False, wheter or not this changeset is open
+                'tag': {} dict of tags,
+                'created_at': timestamp of creation of this changeset
+                'closed_at': timestamp when changeset was closed
+                'comments_count': amount of comments
+                'max_lon': maximum longitude of changes in this changeset
+                'max_lat': maximum latitude of changes in this changeset
+                'min_lon': minimum longitude of changes in this changeset
+                'min_lat': minimum longitude of changes in this changeset
+                'user': username of user that created this changeset,
+                'uid': id of user that created this changeset,
+            }
+        """
+        try:
+            data = self._post(
+                "/api/0.6/changeset/"+str(ChangesetId)+"/unsubscribe",
+                None
+            )
+        except ApiError as e:
+            if e.status == 404:
+                raise NotSubscribedApiError(e.status, e.reason, e.payload)
+            else:
+                raise
+        data = xml.dom.minidom.parseString(data)
+        data = data.getElementsByTagName("osm")[0]
+        data = data.getElementsByTagName("changeset")[0]
+        return self._DomParseChangeset(data)
+
     ##################################################
     # Notes                                          #
     ##################################################
@@ -1626,16 +1761,22 @@ class OsmApi:
                     if i == 5:
                         raise
                     if i != 1:
-                        time.sleep(5)
-                    self._conn = httplib.HTTPConnection(self._api, 80)
+                        self._sleep()
+                    self._conn = self._get_http_connection()
                 else:
                     raise
             except Exception:
                 if i == 5:
                     raise
                 if i != 1:
-                    time.sleep(5)
-                self._conn = httplib.HTTPConnection(self._api, 80)
+                    self._sleep()
+                self._conn = self._get_http_connection()
+
+    def _get_http_connection(self):
+        return httplib.HTTPConnection(self._api, 80)
+
+    def _sleep(self):
+        time.sleep(5)
 
     def _get(self, path):
         return self._http('GET', path, False, None)
@@ -1681,6 +1822,16 @@ class OsmApi:
                 v = (v == "true")
             elif k == "ref":
                 v = int(v)
+            elif k == "comments_count":
+                v = int(v)
+            elif k == "timestamp":
+                v = self._ParseDate(v)
+            elif k == "created_at":
+                v = self._ParseDate(v)
+            elif k == "closed_at":
+                v = self._ParseDate(v)
+            elif k == "date":
+                v = self._ParseDate(v)
             result[k] = v
         return result
 
@@ -1704,6 +1855,21 @@ class OsmApi:
             result.append(int(int(t.attributes["ref"].value)))
         return result
 
+    def _DomGetDiscussion(self, DomElement):
+        """
+        Returns the dictionnary of comments of a DomElement.
+        """
+        result = []
+        try:
+            discussion = DomElement.getElementsByTagName("discussion")[0]
+            for t in discussion.getElementsByTagName("comment"):
+                comment = self._DomGetAttributes(t)
+                comment['text'] = self._GetXmlValue(t, "text")
+                result.append(comment)
+        except IndexError:
+            pass
+        return result
+
     def _DomGetComments(self, DomElement):
         """
         Returns the list of comments of a DomElement.
@@ -1761,6 +1927,8 @@ class OsmApi:
         """
         result = self._DomGetAttributes(DomElement)
         result["tag"] = self._DomGetTag(DomElement)
+        result["discussion"] = self._DomGetDiscussion(DomElement)
+
         return result
 
     def _DomParseNote(self, DomElement):
@@ -1782,10 +1950,16 @@ class OsmApi:
         return result
 
     def _ParseDate(self, DateString):
+        result = DateString
         try:
-            return datetime.strptime(DateString, "%Y-%m-%d %H:%M:%S UTC")
+            result = datetime.strptime(DateString, "%Y-%m-%d %H:%M:%S UTC")
         except:
-            return None
+            try:
+                result = datetime.strptime(DateString, "%Y-%m-%dT%H:%M:%SZ")
+            except:
+                pass
+
+        return result
 
     ##################################################
     # Internal xml builder                           #
